-- ************************************************************************
--  This is the BEEP signal generator
--				(C)Copyright 2014,03,17 by Y nishimura
-- ************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity RtimeClock is
	port
		(
			Hclk		: in  std_logic;	-- Master 36MHz clock
			Reset		: in  std_logic;	-- Master Reset
			Trigb		: in  std_logic;	-- Beep trigger
			Errorb		: in  std_logic;	-- Error Beep
			BeepVol		: in  std_logic_vector(2 downto 0); -- Beep Volume
			Pgate		: out std_logic;	-- Slow gate
			Beep		: out std_logic_vector(7 downto 0)	-- Beep sound
		);
end RtimeClock;

architecture norm of RtimeClock is
	signal	OneK				: std_logic_vector(15 downto 0);
	signal	Beepv,Beepc			: std_logic_vector(7 downto 0);
	signal	Beeps,T1k			: std_logic;
	
begin
	Beep <= Beepv; Pgate <= T1k;
----------------------------------------------------------------------------
--		1kHz counter
----------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then OneK <= "0000000000000000"; T1k <= '0';
		elsif rising_edge(Hclk) then
			if OneK(15 downto 14) = "11"	then OneK <= "0000000000000000"; T1k <= '1';
											else OneK <= OneK + 1; T1k <= '0';
			end if;
		end if;
	end process;

------------------------------------------------------------------------------
--		Beep Time
------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Beepc <= "00000000";
		elsif rising_edge(Hclk) then
			if Beepc(7) = '0'		then
				if Trigb = '1'		then
					if Errorb = '1'	then Beepc <= "10000000";
									else Beepc <= "11000000";
					end if;
				else Beepc <= "00000000";
				end if;
			else
				if T1k = '1'		then Beepc <= Beepc + 1;
									else Beepc <= Beepc;
				end if;
			end if;
		end if;
	end process;

	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Beeps <= '0';
		elsif rising_edge(Hclk) then
			if Beepc(7) = '1'	then
				if Errorb = '1'	then Beeps <= Beepc(1);
								else Beeps <= Beepc(0);
				end if;
			else Beeps <= '0';
			end if;
		end if;
	end process;

------------------------------------------------------------------------------
--		Beep sound
------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Beepv <= "00000000";
		elsif rising_edge(Hclk) then
			if Beepc(7) = '0'	then Beepv <= "00000000";
			else
				case BeepVol is
					when "000" => Beepv <= "00000000";
					when "001" => if Beeps = '1'	then Beepv <= "00000001";
													else Beepv <= "11111111";
								  end if;
					when "010" => if Beeps = '1'	then Beepv <= "00000010";
													else Beepv <= "11111110";
								  end if;
					when "011" => if Beeps = '1'	then Beepv <= "00000100";
													else Beepv <= "11111100";
								  end if;
					when "100" => if Beeps = '1'	then Beepv <= "00001000";
													else Beepv <= "11111000";
								  end if;
					when "101" => if Beeps = '1'	then Beepv <= "00010000";
													else Beepv <= "11110000";
								  end if;
					when "110" => if Beeps = '1'	then Beepv <= "00100000";
													else Beepv <= "11100000";
								  end if;
					when "111" => if Beeps = '1'	then Beepv <= "01000000";
													else Beepv <= "11000000";
								  end if;
					when others => Beepv <= "00000000";   
				end case;
			end if;
		end if;
	end process;

end norm;
